package processModel;

import java.awt.Image;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import javax.swing.JApplet;
import javax.swing.JComponent;

import processView.Externaliser;
import processView.ImagePanel;
import processView.ProcessApplet;
import utilities.AppConstants;

public class SpecialValue
{
	public static final String SPECIAL_VALUE_CV = "Cv";
	public static final String SPECIAL_VALUE_CP = "Cp";
	public static final String SPECIAL_VALUE_R = "R";
	public static final String SPECIAL_VALUE_GAMMA = "\u03B3";
	public static final String SPECIAL_VALUE_N = "n";
	public static final String SPECIAL_VALUE_GAMMA_WORD = "Gamma";
	
	private String name;
	private double value;
	private int id;
	private String imageFile;
	private URL base;
	
	public SpecialValue(String name, double value, int id, URL base, String imageFile)
	{
		this.name = name;
		this.value = value;
		this.id = id;
		this.imageFile = imageFile;
		String baseString = base.toString();
		try {
			this.base = new URL(baseString.substring(0, baseString.lastIndexOf("/")));
		}
		catch (MalformedURLException ex) {
			ex.printStackTrace();
		}
	}
	
	public String toASPIRE_XML(String indent) {
		String xmlStr = "";
		
		xmlStr +=  Externaliser.openInstanceTag(indent, this.getID(), "SpecialValue");
		xmlStr += specialValueASPIRE_XML(indent);
		xmlStr +=  Externaliser.closeInstanceTag(indent);							
		
		return xmlStr;
    }
	
	protected String specialValueASPIRE_XML(String indent) {
		String xmlStr = "";
		
		// Symbol
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 0, getName());		

		// Value
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 1, String.valueOf(getValue()));
		
		return xmlStr;
	}
	
	/**
	 * Creates and returns a JComponent containing the special value image
	 * @param applet JApplet which will contain/display this equation
	 * @return JComponent The component containing the equation image
	 */
	public JComponent imageComponent (ProcessApplet applet){
		Image imgEquation = applet.getImageCache().getImage(imageFile);
		
		JComponent imagePanel = new ImagePanel(imgEquation);
		return imagePanel;
	}
	
	public String toString()
	{
		return name;
	}
	
	public String getName()
	{
		return name;
	}
	
	public double getValue()
	{
		return value;
	}
	
	public int getID()
	{
		return id;
	}
}
