package processModel;

import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.text.DecimalFormat;

import processView.Externaliser;
import utilities.AppConstants;

public class StateUnknown implements Unknown
{
	public enum StateUnknown_Type
	{
		Temperature("Temperature", "T-Unknown"), Pressure("Pressure", "P-Unknown"), Volume("Volume", "V-Unknown");
		
		private String name;
		private String aspireName;
		
		private StateUnknown_Type(String name, String aspireName)
		{
			this.name = name;
			this.aspireName = aspireName;
		}
		
		public String getName() { return name; }
		public String getAspireName() { return aspireName; }
	}
	
	private StateUnknown_Type type;
	private StateElement state;
	private int id;
	private boolean beingWorkedOn = false;
	
	public StateUnknown(StateElement state, int id, StateUnknown_Type type)
	{
		this.type = type;
		this.id = id;
		this.state = state;
	}
	
	public void setUnknownValue(String value, String units)
	{
		if (type.equals(StateUnknown_Type.Temperature)) {
			state.setCalculatedTemperature(value);
			state.temperatureUnit(units);
		}
		else if (type.equals(StateUnknown_Type.Pressure)) {
			state.setCalculatedPressure(value);
			state.pressureUnit(units);
		}
		else if (type.equals(StateUnknown_Type.Volume)) {
			state.setCalculatedVolume(value);
			state.volumeUnit(units);
		}
	}
	
	public String getUnknownValue()
	{
		String result = "";
		if (type.equals(StateUnknown_Type.Temperature)) {
			result = state.temperature();
		}
		else if (type.equals(StateUnknown_Type.Pressure)) {
			result = state.pressure();
		}
		else if (type.equals(StateUnknown_Type.Volume)) {
			result = state.volume();
		}
		return Process.roundNumber(result);
	}
	
	public String getUnknownUnit()
	{
		String result = "";
		if (type.equals(StateUnknown_Type.Temperature)) {
			result = state.getTemperatureUnits();
		}
		else if (type.equals(StateUnknown_Type.Pressure)) {
			result = state.getPressureUnits();
		}
		else if (type.equals(StateUnknown_Type.Volume)) {
			result = state.getVolumeUnits();
		}
		return result;
	}
	
	public void clearUnknownValue()
	{
		if (type.equals(StateUnknown_Type.Temperature)) {
			state.setTemperatureUnknown();
			state.temperatureUnit("");
		}
		else if (type.equals(StateUnknown_Type.Pressure)) {
			state.setPressureUnknown();
			state.pressureUnit("");
		}
		else if (type.equals(StateUnknown_Type.Volume)) {
			state.setVolumeUnknown();
			state.volumeUnit("");
		}
	}
	
	public int getID()
	{
		return id;
	}
	
	public String toASPIRE_XML(String indent) {
		String xmlStr = "";
		
		xmlStr += Externaliser.openInstanceTag(indent, this.getID(), type.getAspireName());
		xmlStr += stateUnknownASPIRE_XML(indent);
		xmlStr += Externaliser.closeInstanceTag(indent);							
		
		return xmlStr;
    }
	
	protected String stateUnknownASPIRE_XML(String indent) {
		String xmlStr = "";
		
		// Value
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 1, getUnknownValue());
		
		// Unit
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 2, getUnknownUnit());
		
		// State Label
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 2, state.label());
		
		return xmlStr;
	}
	
	public String toString()
	{
		return "State " + state.label() + ": " + type.getName();
	}
}
