/**
 * 
 */
package processModel;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;
import java.util.Observable;

import processModel.TransitionUnknown.TransitionUnknown_Type;
import processView.Externaliser;

import utilities.AppConstants;

/**
 * @author Administrator
 *
 */
public abstract class Transition implements Element, Moveable {

		private String label="";
		private Line2D.Double line;
		private Boolean selected = false;
		private List <TransitionHandle> handles = new ArrayList<TransitionHandle>();
		private Point beginPoint = null;
		private Point endPoint = null;
		private String delta = "";
		private String heat = "";
		private String work = "";
		private String workUnit = "";
		private String heatUnit = "";
		private String deltaUnit = "";
		private boolean deltaCalculated = false;
		private boolean heatCalculated = false;
		private boolean workCalculated = false;
		private List<Unknown> calculatedValues = null;
		protected int id = -1;
		public String type;
		
		public Transition(String type) {
			this.type = type;		
		}

			
		/**
		 * 
		 */
		public Transition(Point p1, Point p2) {
			beginPoint = p1;
			endPoint = p2;
			createTransitionShape();
			createHandles();
		}
		
		public boolean selected() {
			return selected;
		}
		
		protected abstract void createTransitionShape ();
		
		/* (non-Javadoc)
		 * @see processModel.Element#paint(java.awt.Graphics2D)
		 */
		public abstract void paint(Graphics2D g2);
		
		public void setOrigin(Point p) {
			beginPoint = new Point (p);
			endPoint = new Point ((int)p.getX()+ AppConstants.MINIMUM_TRANSITION_SIZE, (int) p.getY());	
			createTransitionShape();
			createHandles();
		}
	
		/* (non-Javadoc)
		 * @see processModel.Element#setEndPoint(java.awt.Point)
		 */
		public void setEndPoint(Point p) {
			endPoint = p;
			createTransitionShape();
			createHandles();
		}
		
		/**
		 * @return true if the receiver is not fully joined to states.
		 */
		public boolean isNotJoined()
		{
			for (TransitionHandle handle : handles) {
				if (!handle.isConnected())
				{
					return true;
				}
			}
			return false;
		}
	
		public void setBeginPoint(Point p) {
			beginPoint = p;
			if (endPoint != null){
				createTransitionShape();
				createHandles();
			}
		}
		
		protected double getLineLength()
		{
			return Math.sqrt(
					Math.pow((beginPoint().x - endPoint().x), 2) +
					Math.pow((beginPoint().y - endPoint().y), 2));
		}
		
		public String toString() {
			return this.getClass().getCanonicalName() + " labelled " + label;
		}
		
		public List<TransitionHandle> getHandles() {
			return handles;
		}
		
		public  void setHandles(List<TransitionHandle> handles) {
			this.handles = handles;
		}
		
		private void createHandles(){
			// first handle on first point
			if (endPoint == null){
				endPoint = beginPoint;
			}
			if (beginPoint != null && endPoint != null){
				TransitionHandle handle1 = new TransitionHandle(beginPoint, this);
				TransitionHandle handle2 = new TransitionHandle(endPoint, this);
				handle1.setID(id+1);
				handle2.setID(id+2);
	
				// add cursors
				handle1.setCursorType(Cursor.W_RESIZE_CURSOR);
				handle1.setCursorType(Cursor.E_RESIZE_CURSOR);
				
				// Check if the list of handles contains the handle
				// if it does, update the current handle
				// otherwise create new handles and add them to the list
			
				TransitionHandle currentHandle1 = null;
				TransitionHandle currentHandle2 = null;
				
				try {
					currentHandle1 = handles.get(0);
					currentHandle2 = handles.get(1);
				}catch (Exception e){
					// No handles exist; continue as is
				}
				
				if (currentHandle1 != null)
					currentHandle1.move(beginPoint);
				else
					handles.add(0, handle1);
				
				if (currentHandle2 != null)
					currentHandle2.move(endPoint);
				else 
					handles.add(1,handle2);
					
				for(TransitionHandle handle: handles)
					handle.addObs(this);
	
			}
			
	}
		public void select(boolean select) {
				for (TransitionHandle handle:handles)
					handle.select(select);
				this.selected = select;
		}
	
		// The observable should be a transition Handle
		public void update(Observable objTransitionHandle, Object intTypeOfChange) {
			int typeOfChange = 0;
			TransitionHandle transitionHandle = null;
			try {
				typeOfChange = (Integer)intTypeOfChange;
				transitionHandle = (TransitionHandle)objTransitionHandle;
				
				if (typeOfChange == AppConstants.RESIZE_OWNER){
					if (handles.get(0) == transitionHandle)
						setBeginPoint(transitionHandle.origin());
					if (handles.get(1) == transitionHandle)
						setEndPoint (transitionHandle.origin());	
				}
			}catch (Exception e){}
			}
		
		//public abstract Transition pointIsInTransition (Point p);
		
		public abstract Element pointIsInElement(Point p);
		
		public Handle pointIsInHandle (Point p){
	
			Handle selectedHandle = null;
			
			for (Handle tempHandle : handles){
				if (tempHandle.pointIsInHandle(p) != null){
					return selectedHandle = tempHandle.pointIsInHandle(p);
				}
			}
			return selectedHandle;
		}
		
		public void prepareForDeletion() {
			for (TransitionHandle handle:handles){
				handle.prepareForDeletion();
				handle = null;
			}
			handles = null;
		}
		
		public void label(String l) {
			label = l;
		}
		
		public int getID ()
		{
			return id;
		}
		
		public void setID(int id)
		{
			this.id = id;
		}
		
		/*public boolean deleted ()
		{
			return id == -1;
		}*/
		
		public Point beginPoint(){return beginPoint;}
		public Point endPoint(){return endPoint;}
		public Line2D line() {return line;}
		public void line(Line2D.Double line){this.line = line;}
		
		public void work(String work){this.work = work.trim();}
		public void heat(String heat){this.heat = heat.trim();}
		public void delta(String delta){this.delta = delta.trim();}
		public void workUnit(String workUnit){this.workUnit = workUnit.trim();}
		public void heatUnit(String heatUnit){this.heatUnit = heatUnit.trim();}
		public void deltaUnit(String deltaUnit){this.deltaUnit = deltaUnit.trim();}
		
		public void setCalculatedDelta(String delta)
		{
			this.delta = delta.trim();
			deltaCalculated = true;
		}
		
		public void setCalculatedWork(String work)
		{
			this.work = work.trim();
			workCalculated = true;
		}
		
		public void setCalculatedHeat(String heat)
		{
			this.heat = heat.trim();
			heatCalculated = true;
		}
		
		public boolean deltaCalculated()
		{
			return deltaCalculated;
		}
		
		public boolean workCalculated()
		{
			return workCalculated;
		}
		
		public boolean heatCalculated()
		{
			return heatCalculated;
		}
		
		public String work() {
			if (workUnknown()) {
				return "";
			} else {
				return work;	
			}		
		}
		
		public String heat()
		{
			if (heatUnknown()) {
				return "";
			} else {
				return heat;	
			}
		}
		
		public String delta()
		{
			if (deltaUnknown()) {
				return "";
			} else {
				return delta;	
			}
		}
		
		public boolean workUnknown() {
			return this.work == null;
		}
		
		public boolean heatUnknown() {
			return this.heat == null;
		}
		
		public boolean deltaUnknown() {
			return this.delta == null;
		}
		
		public void setWorkUnknown() {
			this.work = null;
		}
		
		public void setHeatUnknown() {
			this.heat = null;
		}
		
		public void setDeltaUnknown() {
			this.delta = null;
		}
		
		public List<Unknown> getUnknowns(){
			if (calculatedValues == null) {
				calculatedValues = new ArrayList<Unknown>();
				calculatedValues.add(new TransitionUnknown(this, Process.getNextID(), TransitionUnknown_Type.Work));
				calculatedValues.add(new TransitionUnknown(this, Process.getNextID(), TransitionUnknown_Type.Heat));
				calculatedValues.add(new TransitionUnknown(this, Process.getNextID(), TransitionUnknown_Type.DeltaU));
			}
			List<Unknown> unknowns = new ArrayList<Unknown>(calculatedValues);
			ListIterator<Unknown> iter = unknowns.listIterator();
			while (iter.hasNext())
			{
				if (!iter.next().getUnknownValue().equals("")){
					iter.remove();
				}
			}
			return unknowns;
	    }
		
		public List<Unknown> getCalculatedValues()
		{
			List<Unknown> result = new ArrayList<Unknown>(calculatedValues);
			ListIterator<Unknown> iter = result.listIterator();
			while (iter.hasNext())
			{
				Unknown next = iter.next();
				if (next.getUnknownValue().equals("")) {
					iter.remove();
					System.out.println("Unknown Removed: " + next.getID());
				}
				else {
					System.out.println("Unknown Kept: " + next.getID());
				}
			}
			return result;
		}
		
		public String label(){return this.label;}
		
		public String getWorkUnits() {
			return workUnit;
		}
		
		public String getHeatUnits() {
			return heatUnit;
		}
		
		public String getDeltaUnits() {
			return deltaUnit;
		}
		
		public boolean isExpansion()
		{
			return endPoint.x > beginPoint.x && endPoint.y > beginPoint.y;
		}
		
		public boolean isCompression()
		{
			return endPoint.x < beginPoint.x && endPoint.y < beginPoint.y;
		}
		
		public boolean isInvalidExpansionOrCompression()
		{
			return 	endPoint.x < beginPoint.x && endPoint.y > beginPoint.y ||
					endPoint.x > beginPoint.x && endPoint.y < beginPoint.y;
		}
		
		public abstract void move(Point p);
		// Temporary decision: Only States can move, not transitions.
		// When states move, they will resize connected transitions.
		
		protected String TransitionASPIRE_XML(String indent) {
			String xmlStr = "";
			
			// Work
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 1, this.work());
			// Units
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 3, this.getWorkUnits());
			// Work Unknown
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 2, String.valueOf(this.workUnknown()));
			
			// Heat
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 4, this.heat());
			// Units
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 6, this.getHeatUnits());
			// Heat Unknown
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 5, String.valueOf(this.heatUnknown()));
			
			// Delta
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 7, this.delta());
			// Units
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 9, this.getDeltaUnits());
			// Delta Unknown
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 8, String.valueOf(this.deltaUnknown()));
			
			
			// Label - 0
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 0, this.label());	
			
			//		 ConnectedToStart - 4
			if((getHandles().get(0).getPartner()) != null){
				xmlStr +=  Externaliser.openRelatedSlotTag(indent + AppConstants.Sx2, 10);			
				xmlStr += Externaliser.relatedInstanceTag(indent + AppConstants.Sx4, ((StateHandle)(getHandles().get(0).getPartner())).getParent().getID());
				xmlStr +=  Externaliser.closeRelatedSlotTag(indent + AppConstants.Sx2);
			}
			
			//		 ConnectedToEnd - 5
			if((getHandles().get(1).getPartner()) != null){
				xmlStr +=  Externaliser.openRelatedSlotTag(indent + AppConstants.Sx2, 11);
				xmlStr += Externaliser.relatedInstanceTag(indent + AppConstants.Sx4, ((StateHandle)(getHandles().get(1).getPartner())).getParent().getID());
				xmlStr +=  Externaliser.closeRelatedSlotTag(indent + AppConstants.Sx2);
			}
			
			return xmlStr;
		}
		
		protected String transitionXMLContent(String indent) {
	    	String xmlStr = "";
	    	
	    	//	end points
			xmlStr += getHandles().get(0).toXML(indent + AppConstants.Sx2);			
			xmlStr += getHandles().get(1).toXML(indent + AppConstants.Sx2);
			
			//	Label and magnitudes
			if (!"".equals(this.label)) {
				xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_LABEL + AppConstants.S + AppConstants.ATT_VALUE +
						Externaliser.quoted(this.label()));
			}
			
			xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_WORK 
					+ AppConstants.S + AppConstants.ATT_VALUE + Externaliser.quoted(this.work()) 
					+ AppConstants.S + AppConstants.ATT_UNKNOWN + Externaliser.quoted(this.workUnknown()));
		
			if (!"".equals(this.workUnit)) {
				xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_UNITS + AppConstants.S + AppConstants.ATT_VALUE +
						Externaliser.quoted(this.getWorkUnits()));
			}
			
			xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_HEAT 
					+ AppConstants.S + AppConstants.ATT_VALUE + Externaliser.quoted(this.heat()) 
					+ AppConstants.S + AppConstants.ATT_UNKNOWN + Externaliser.quoted(this.heatUnknown()));
		
			if (!"".equals(this.heatUnit)) {
				xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_UNITS + AppConstants.S + AppConstants.ATT_VALUE +
						Externaliser.quoted(this.getHeatUnits()));
			}
			
			xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_DELTA 
					+ AppConstants.S + AppConstants.ATT_VALUE + Externaliser.quoted(this.delta()) 
					+ AppConstants.S + AppConstants.ATT_UNKNOWN + Externaliser.quoted(this.deltaUnknown()));
		
			if (!"".equals(this.deltaUnit)) {
				xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_UNITS + AppConstants.S + AppConstants.ATT_VALUE +
						Externaliser.quoted(this.getDeltaUnits()));
			}
			
	    	return xmlStr;
		}
	}
