package processView;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.FocusTraversalPolicy;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import javax.swing.SpringLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import utilities.SpringUtilities;
import javax.swing.SpringLayout;

import controls.UnitComboBox;



import utilities.AppUtils;

import utilities.AppConstants;

import processModel.StateElement;
import processModel.Process;
import processModel.StateHandle;
import processModel.Transition;

public class StateDialogMenu extends DialogMenu {

	private StateElement state = null;
	private String pressure = "";
	private String volume = "";
	private String temperature = "";
	private String delta = "";
	private String pressureUnit = "";
	private String volumeUnit = "";
	private String temperatureUnit = "";
	private String deltaUnit = "";
	private JTextField txtLabel;
	private JTextField txtPressure;
	private JTextField txtVolume;
	private JTextField txtTemperature;
	private UnitComboBox cmbPressureUnit;
	private UnitComboBox cmbVolumeUnit;
	private UnitComboBox cmbTemperatureUnit;
	private JTextField txtDelta;
	private JCheckBox chkPressureUnknown;
	private JCheckBox chkVolumeUnknown;
	private JCheckBox chkTemperatureUnknown;
	private Frame owner;

	private static final long serialVersionUID = -7891430862450273942L;

	public StateDialogMenu (Frame owner, String title, StateElement state, Process process){
		super(owner, title, process);
		this.state = state;
		this.label(state.label());
		this.temperature(state.temperature());
		this.volume(state.volume());
		this.pressure(state.pressure());
		this.delta(state.delta());
		this.temperatureUnit(state.getTemperatureUnits());
		this.volumeUnit(state.getVolumeUnits());
		this.pressureUnit(state.getPressureUnits());
		this.setSize(200, 600);
		createGUI();
		setFocusTraversalPolicy(new StateDialogFocusTraversalPolicy());
	}

	
	public void volume(String volume){
		if (volume == null) 
			this.volume = "";
		else 
			this.volume = volume.trim();
		}
	
	public void temperature(String temp){
		if (temp == null)
			this.temperature = "";
		else
			this.temperature = temp.trim();
		}
	
	public void pressure (String pressure){
		if (pressure == null)
			this.pressure = "";
		else
			this.pressure = pressure.trim();
		}
	
	public void delta (String delta){
		if (delta == null)
			this.delta = "";
		else
			this.delta = delta.trim();
		}
	
	public void volumeUnit(String volumeUnit){
		if (volumeUnit == null) 
			this.volumeUnit = "";
		else 
			this.volumeUnit = volumeUnit.trim();
		}
	
	public void temperatureUnit(String tempUnit){
		if (tempUnit == null)
			this.temperatureUnit = "";
		else
			this.temperatureUnit = tempUnit.trim();
		}
	
	public void pressureUnit (String pressureUnit){
		if (pressureUnit == null)
			this.pressureUnit = "";
		else
			this.pressureUnit = pressureUnit.trim();
		}
	
	public void deltaUnit (String deltaUnit){
		if (deltaUnit == null)
			this.deltaUnit = "";
		else
			this.deltaUnit = deltaUnit.trim();
		}
	protected void createGUI(){
		//		 The Panels		
		JPanel contentPane = new JPanel();
		AppUtils.restrictSize(contentPane, new Dimension(400, 220));
		JPanel pnlMain = new JPanel(new SpringLayout());
		
		JLabel lbl1 = new JLabel(" Property");
		lbl1.setFont(AppConstants.arialBold);		
		pnlMain.add(lbl1);
		JLabel lbl2 = new JLabel("Value");
		lbl2.setFont(AppConstants.arialBold);
		pnlMain.add(lbl2);
		JLabel lbl4 = new JLabel("Units");
		lbl4.setFont(AppConstants.arialBold);
		pnlMain.add(lbl4);
		JLabel lbl3 = new JLabel("Unknown");
		lbl3.setFont(AppConstants.arialBold);
		pnlMain.add(lbl3);
		
		// The Components
		Font font = new Font ("Arial", Font.PLAIN, 12);
		
		JLabel lblLabel = new JLabel(" Label ");
		lblLabel.setFont(font);
		pnlMain.add(lblLabel);
		txtLabel = new JTextField(label(),10);
        pnlMain.add(txtLabel);
        pnlMain.add(Box.createHorizontalStrut(5));
        pnlMain.add(Box.createHorizontalStrut(5));
        
        JLabel lblPressure = new JLabel(" Pressure ");        
		lblPressure.setFont(font);
        pnlMain.add(lblPressure);
        txtPressure = new JTextField(pressure, 10);
        pnlMain.add(txtPressure);    
        cmbPressureUnit = new UnitComboBox(process);
        cmbPressureUnit.setSelectedItemByString(pressureUnit);
        
        
        pnlMain.add(cmbPressureUnit);  
        
        chkPressureUnknown = new JCheckBox();
        if(state.pressureUnknown())
        {
        	chkPressureUnknown.setSelected(true);
        	txtPressure.setEnabled(false);
        	cmbPressureUnit.setEnabled(false);
        	txtPressure.setText("");
			cmbPressureUnit.setSelectedIndex(0);
        }
        
        chkPressureUnknown.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				txtPressure.setEnabled(!txtPressure.isEnabled());
				cmbPressureUnit.setEnabled(!cmbPressureUnit.isEnabled());
				if(!txtPressure.isEnabled()) {
					txtPressure.setText("");
					cmbPressureUnit.setSelectedIndex(0);
				}
			}			
		});
        pnlMain.add(chkPressureUnknown);
		
        JLabel lblVolume = new JLabel(" Volume ");
        lblVolume.setFont(font);
        pnlMain.add(lblVolume);
        txtVolume = new JTextField(volume,10);
        pnlMain.add(txtVolume);
        cmbVolumeUnit = new UnitComboBox(process);
        cmbVolumeUnit.setSelectedItemByString(volumeUnit);
        pnlMain.add(cmbVolumeUnit);  
        
        chkVolumeUnknown = new JCheckBox();
        chkVolumeUnknown = new JCheckBox();
        if(state.volumeUnknown())
        {
        	chkVolumeUnknown.setSelected(true);
        	txtVolume.setEnabled(false);
        	cmbVolumeUnit.setEnabled(false);
        	txtVolume.setText("");
			cmbVolumeUnit.setSelectedIndex(0);
        	
        }
       
        chkVolumeUnknown.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				txtVolume.setEnabled(!txtVolume.isEnabled());
				cmbVolumeUnit.setEnabled(!cmbVolumeUnit.isEnabled());
				if(!txtVolume.isEnabled()) {
					txtVolume.setText("");
					cmbVolumeUnit.setSelectedIndex(0);
				}
			}			
		});
        pnlMain.add(chkVolumeUnknown);
		
        JLabel lblTemperature = new JLabel(" Temperature ");
        lblTemperature.setFont(font);
        pnlMain.add(lblTemperature);
        txtTemperature = new JTextField(temperature,10);
        pnlMain.add(txtTemperature);
        cmbTemperatureUnit = new UnitComboBox(process);
        cmbTemperatureUnit.setSelectedItemByString(temperatureUnit);
        pnlMain.add(cmbTemperatureUnit);  
        
        chkTemperatureUnknown = new JCheckBox();
        if(state.temperatureUnknown())
        {
        	chkTemperatureUnknown.setSelected(true);
        	txtTemperature.setEnabled(false);
        	cmbTemperatureUnit.setEnabled(false);
        	txtTemperature.setText("");
			cmbTemperatureUnit.setSelectedIndex(0);
        	
        }
        
        chkTemperatureUnknown.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				txtTemperature.setEnabled(!txtTemperature.isEnabled());
				cmbTemperatureUnit.setEnabled(!cmbTemperatureUnit.isEnabled());
				if(!txtTemperature.isEnabled()) {
					txtTemperature.setText("");
					cmbTemperatureUnit.setSelectedIndex(0);
				}
			}			
		});
        pnlMain.add(chkTemperatureUnknown);
		
		SpringUtilities.makeCompactGrid(pnlMain, 5, 4, 3, 3, 7, 7);
		pnlMain.setAlignmentX(Component.CENTER_ALIGNMENT);

		contentPane.add(pnlMain);
		contentPane.add(createButtons(), BorderLayout.SOUTH);
		
		this.add(contentPane);
		
		addWindowListener(new WindowAdapter(){
		       //capture the window closing event i.e clicking 'X'
		       public void windowClosing(WindowEvent evt){
		    	   if(process.newState!=null) {
		            	process.removeState(state);
		                dispose();
		                }
	            	else dispose();
	            	}
		       });
	}
	
	protected JComponent createButtons() {
        JPanel panel = new JPanel(new FlowLayout(FlowLayout.TRAILING));
        
        JButton btnCancel = new JButton("Cancel");
        
        // final properties - for use in the ActionListener
        final DialogMenu dialog = this;

        btnCancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	if(process.newState!=null) {
	            	process.removeState(state);
	                dialog.setVisible(false);
	                dialog.dispose();
	                }
            	else {
            		dialog.setVisible(false);
	                dialog.dispose();
	                }
            	}    	
            });
        
        panel.add(btnCancel);

        JButton btnOK = new JButton("OK");
        btnOK.addActionListener(new ActionListener(){
        	public void actionPerformed(ActionEvent e){
         		boolean changed = false;
         		boolean closeDialog = true;
         		
                String labelTxt = txtLabel.getText().trim();
        		String pressureTxt = txtPressure.getText().trim();	
        		String volumeTxt = txtVolume.getText().trim();
        		String temperatureTxt = txtTemperature.getText().trim();
        		//String deltaTxt = txtDelta.getText().trim();
        		
        		if(!process.isLabelValid(labelTxt)){
        			changed = false;
        			closeDialog = false;
        			JOptionPane.showMessageDialog(dialog,
                    "The label you have chosen is not a single digit. Diagram labels should be unique single digits.");
        		}
        		else if (!labelTxt.equals(state.label())){
        			System.out.println("They are not equal!");
        			
        			// check the new label is unique
        			if (process.isLabelUsed(labelTxt)){
        				changed = false;
        				closeDialog = false;
        				JOptionPane.showMessageDialog(dialog,
                        "The label you have chosen is not unique. Diagram labels should be unique.");
        			}else {
        				changed = true;
        				state.label(labelTxt);
        			}
        		}
        		        			
        		if (chkPressureUnknown.isSelected()) {
        				state.setPressureUnknown();
        			} else {
        				if (pressureTxt.equals("") && state.pressureUnknown())
        				{
        					// Special case for de-selecting the unknown box
        					// TODO: Should there be some validation here?
        					state.pressure("");
        				}
        				else if (!pressureTxt.equals(pressure)){
        					state.pressure(pressureTxt);
        					
	        				for(StateHandle handle: state.getHandles()){
	    						for(Transition transition: process.getTransitionsAt(handle.origin())){
	    							if (!process.checkTransitionPressureIntegrity(transition)){
	    								state.pressure(pressure);
	    								changed=true;
	    								JOptionPane.showMessageDialog(owner, "Please check the states you're attempting to connect.  The position of these states in the diagram is not consistent with the volume and/or pressure associated with them. ");
	    								break;
	    							}        								
	    						}
	    					}
	        				
	        				if(changed==false){
	        					changed = true;
		        				state.pressure(pressureTxt);
		        				}
        			}        			
        		}
        		
        		if (chkVolumeUnknown.isSelected()) {
        				state.setVolumeUnknown();
        			} else {
        				if (volumeTxt.equals("") && state.volumeUnknown())
        				{
        					// Special case for de-selecting the unknown box
        					// TODO: Should there be some validation here?
        					state.volume("");
        				}
        				else if (!volumeTxt.equals(volume)) { 
        					state.volume(volumeTxt);
        					for(StateHandle handle: state.getHandles()){
        						for(Transition transition: process.getTransitionsAt(handle.origin())){
        							if (!process.checkTransitionVolumeIntegrity(transition)){
        								System.out.println("Violated Inside Volume Properties"); 
        								state.volume(volume);
        								changed=true;
        								JOptionPane.showMessageDialog(owner, "Please check the states you're attempting to connect.  The position of these states in the diagram is not consistent with the volume and/or pressure associated with them. ");
        								break;
        							}        								
        						}
        					}
        					
        					if(changed==false){
        						changed = true;
        						state.volume(volumeTxt);
        						}
        			} 
        		}
        		
        		if (chkTemperatureUnknown.isSelected()) {
        				state.setTemperatureUnknown();
        			} else {
        				if (temperatureTxt.equals("") && state.temperatureUnknown())
        				{
        					// Special case for de-selecting the unknown box
        					state.temperature("");
        				}
        				else if (!temperatureTxt.equals(temperature)){
                			changed = true;
                			state.temperature(temperatureTxt);
        			}
				}
        		
        		if (!cmbTemperatureUnit.equals(temperatureUnit)){
        			state.temperatureUnit(cmbTemperatureUnit.getSelectedItemString());
        		}
        		if (!cmbPressureUnit.equals(pressureUnit)){
        			state.pressureUnit(cmbPressureUnit.getSelectedItemString());
        		}
        		if (!cmbVolumeUnit.equals(volumeUnit)){
        			state.volumeUnit(cmbVolumeUnit.getSelectedItemString());
        		}
        		if (closeDialog){
        			dialog.setVisible(false);
        			dialog.dispose();
        		}
        	}
        });
        
        panel.add(btnOK); 

        //Match the SpringLayout's gap, subtracting 5 to make
        //up for the default gap FlowLayout provides.
        panel.setBorder(BorderFactory.createEmptyBorder(0, 0,
                                                5, 5));
        return panel;
    }
	
	class StateDialogFocusTraversalPolicy extends FocusTraversalPolicy
	{
		private List<Component> componentsList;
		
		public StateDialogFocusTraversalPolicy()
		{
			componentsList = new ArrayList<Component>();
			componentsList.add(txtPressure);
			componentsList.add(cmbPressureUnit);
			componentsList.add(txtVolume);
			componentsList.add(cmbVolumeUnit);
			componentsList.add(txtTemperature);
			componentsList.add(cmbTemperatureUnit);
		}
		
		public Component getComponentAfter(Container container, Component component) {
			int currentIndex = componentsList.indexOf(component);
			for (int i = 1; i < componentsList.size(); i++) {
				int index = (currentIndex+i+componentsList.size()) % componentsList.size();
				Component nextComponent = componentsList.get(index);
				if (nextComponent.isEnabled()) {
					return nextComponent;
				}
			}
			return null;
		}

		public Component getComponentBefore(Container container, Component component) {
			int currentIndex = componentsList.indexOf(component);
			for (int i = 1; i < componentsList.size(); i++) {
				int index = (currentIndex-i+componentsList.size()) % componentsList.size();
				Component prevComponent = componentsList.get(index);
				if (prevComponent.isEnabled()) {
					return prevComponent;
				}
			}
			return null;
		}

		public Component getDefaultComponent(Container container) {
			for (Component comp : componentsList) {
				if (comp.isEnabled()) {
					return comp;
				}
			}
			return componentsList.get(0);
		}

		public Component getFirstComponent(Container container) {
			return componentsList.get(0);
		}

		public Component getLastComponent(Container container) {
			return componentsList.get(componentsList.size()-1);
		}
	}
	
	public static void main(String[] args)
	{
		System.out.println((1+10)%10);
	}
}